@@ -241,6 +241,7 @@ signon_identity_dispose (GObject *object)
     if (priv->cancellable)
     {
         g_cancellable_cancel (priv->cancellable);
+        g_object_unref (priv->cancellable);
         priv->cancellable = NULL;
     }
 
@@ -478,7 +479,7 @@ identity_new_cb (GObject *object, GAsyncResult *res,
 {
     SignonIdentity *identity = (SignonIdentity*)userdata;
     SsoAuthService *proxy = SSO_AUTH_SERVICE (object);
-    gchar *object_path;
+    gchar *object_path = NULL;
     GError *error = NULL;
 
     g_return_if_fail (identity != NULL);
@@ -490,6 +491,7 @@ identity_new_cb (GObject *object, GAsyncResult *res,
                                                         &error);
     SIGNON_RETURN_IF_CANCELLED (error);
     identity_registered (identity, object_path, NULL, error);
+    g_free (object_path);
 }
 
 static void
@@ -498,7 +500,7 @@ identity_new_from_db_cb (GObject *object, GAsyncResult *res,
 {
     SignonIdentity *identity = (SignonIdentity*)userdata;
     SsoAuthService *proxy = SSO_AUTH_SERVICE (object);
-    gchar *object_path;
+    gchar *object_path = NULL;
     GVariant *identity_data;
     GError *error = NULL;
 
@@ -512,6 +514,7 @@ identity_new_from_db_cb (GObject *object, GAsyncResult *res,
                                                &error);
     SIGNON_RETURN_IF_CANCELLED (error);
     identity_registered (identity, object_path, identity_data, error);
+    g_free (object_path);
 }
 
 static void
@@ -817,11 +820,6 @@ identity_store_credentials_reply (GObject *object, GAsyncResult *res,
     sso_identity_call_store_finish (proxy, &id, res, &error);
     SIGNON_RETURN_IF_CANCELLED (error);
 
-    if (cb_data->cb)
-    {
-        (cb_data->cb) (cb_data->self, id, error, cb_data->user_data);
-    }
-
     if (error == NULL)
     {
         g_return_if_fail (priv->identity_info == NULL);
@@ -845,6 +843,11 @@ identity_store_credentials_reply (GObject *object, GAsyncResult *res,
         priv->removed = FALSE;
     }
 
+    if (cb_data->cb)
+    {
+        (cb_data->cb) (cb_data->self, id, error, cb_data->user_data);
+    }
+
     g_clear_error(&error);
     g_slice_free (IdentityStoreCredentialsCbData, cb_data);
 }
