@@ -26,6 +26,9 @@
  * @example check_signon.c
  * Shows how to initialize the framework.
  */
+
+#define SIGNON_DISABLE_DEPRECATION_WARNINGS
+
 #include "libsignon-glib/signon-internals.h"
 #include "libsignon-glib/signon-auth-service.h"
 #include "libsignon-glib/signon-auth-session.h"
@@ -36,6 +39,7 @@
 #include <check.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 static GMainLoop *main_loop = NULL;
 static SignonIdentity *identity = NULL;
@@ -357,6 +361,8 @@ START_TEST(test_auth_session_query_mechanisms_nonexisting)
     g_main_loop_run (main_loop);
 
     g_free(patterns[0]);
+    g_free(patterns[1]);
+    g_free(patterns[2]);
     g_object_unref(idty);
 
     end_test ();
@@ -478,8 +484,7 @@ START_TEST(test_auth_session_process)
                                "mech1",
                                test_auth_session_process_cb,
                                sessionData);
-    if(!main_loop)
-        main_loop = g_main_loop_new (NULL, FALSE);
+    main_loop = g_main_loop_new (NULL, FALSE);
 
 
     g_main_loop_run (main_loop);
@@ -517,7 +522,162 @@ START_TEST(test_auth_session_process)
     g_free(passwordVa);
     g_free(passwordKey);
 
+    end_test ();
+}
+END_TEST
 
+static void
+test_auth_session_process_failure_cb (GObject *source_object,
+                                      GAsyncResult *res,
+                                      gpointer user_data)
+{
+    SignonAuthSession *auth_session = SIGNON_AUTH_SESSION (source_object);
+    GVariant *v_reply;
+    GError **error = user_data;
+
+    fail_unless (SIGNON_IS_AUTH_SESSION (source_object));
+
+    v_reply = signon_auth_session_process_finish (auth_session, res, error);
+    fail_unless (v_reply == NULL);
+
+    g_main_loop_quit (main_loop);
+}
+
+START_TEST(test_auth_session_process_failure)
+{
+    SignonAuthSession *auth_session;
+    GVariantBuilder builder;
+    GVariant *session_data;
+    GError *error = NULL;
+
+    g_debug("%s", G_STRFUNC);
+
+    g_type_init ();
+
+    auth_session = signon_auth_session_new (0, "nonexisting-method", &error);
+    fail_unless (auth_session != NULL, "Cannot create AuthSession object");
+    fail_unless (error == NULL);
+
+    g_variant_builder_init (&builder, G_VARIANT_TYPE_VARDICT);
+    g_variant_builder_add (&builder, "{sv}",
+                           "key", g_variant_new_string ("value"));
+
+    session_data = g_variant_builder_end (&builder);
+
+    signon_auth_session_process_async (auth_session,
+                                       session_data,
+                                       "mech1",
+                                       NULL,
+                                       test_auth_session_process_failure_cb,
+                                       &error);
+
+    main_loop = g_main_loop_new (NULL, FALSE);
+    g_main_loop_run (main_loop);
+    fail_unless (error != NULL);
+    fail_unless (error->domain == SIGNON_ERROR);
+    fail_unless (error->code == SIGNON_ERROR_METHOD_NOT_KNOWN);
+
+    g_object_unref (auth_session);
+
+    end_test ();
+}
+END_TEST
+
+static void
+test_auth_session_process_after_store_cb (SignonAuthSession *self,
+                                          GHashTable *reply,
+                                          const GError *error,
+                                          gpointer user_data)
+{
+    GValue *v_username;
+
+    if (error != NULL)
+    {
+        fail("Got error: %s", error->message);
+        g_main_loop_quit (main_loop);
+        return;
+    }
+
+    fail_unless (reply != NULL, "The result is empty");
+
+    v_username = g_hash_table_lookup(reply,
+                                     SIGNON_SESSION_DATA_USERNAME);
+
+    fail_unless (g_strcmp0 (g_value_get_string (v_username), "Nice user") == 0,
+                 "Wrong value of username");
+
+    g_hash_table_unref (reply);
+    g_object_unref (self);
+
+    g_main_loop_quit (main_loop);
+}
+
+static void
+test_auth_session_process_after_store_start_session(SignonIdentity *self,
+                                                    guint32 id,
+                                                    const GError *error,
+                                                    gpointer user_data)
+{
+    GError *err = NULL;
+
+    if (error != NULL)
+    {
+        g_warning ("%s %d: %s", G_STRFUNC, __LINE__, error->message);
+        fail();
+        g_main_loop_quit (main_loop);
+        return;
+    }
+
+    fail_unless (id > 0);
+
+    SignonAuthSession *auth_session =
+        signon_identity_create_session (self,
+                                        "ssotest",
+                                        &err);
+
+    fail_unless (auth_session != NULL, "Cannot create AuthSession object");
+    if (err != NULL)
+    {
+        fail ("Got error: %s", err->message);
+        g_clear_error (&err);
+    }
+
+    GHashTable *session_data = g_hash_table_new (g_str_hash,
+                                                 g_str_equal);
+
+    signon_auth_session_process (auth_session,
+                                 session_data,
+                                 "mech1",
+                                 test_auth_session_process_after_store_cb,
+                                 NULL);
+}
+
+START_TEST(test_auth_session_process_after_store)
+{
+    SignonIdentityInfo *info;
+    SignonIdentity *identity;
+
+    g_debug("%s", G_STRFUNC);
+
+    g_type_init();
+    main_loop = g_main_loop_new (NULL, FALSE);
+
+    identity = signon_identity_new ();
+    fail_unless (SIGNON_IS_IDENTITY (identity),
+                 "Failed to initialize the Identity.");
+
+    info = signon_identity_info_new ();
+    signon_identity_info_set_username (info, "Nice user");
+
+    signon_identity_store_credentials_with_info (identity,
+                                                 info,
+                                                 test_auth_session_process_after_store_start_session,
+                                                 NULL);
+    g_main_loop_run (main_loop);
+
+    g_object_unref (identity);
+
+    end_test ();
 }
 END_TEST
 
@@ -567,9 +727,6 @@ new_identity()
     GHashTable *methods;
     guint id = 0;
 
-    if (main_loop == NULL)
-        main_loop = g_main_loop_new (NULL, FALSE);
-
     identity = signon_identity_new(NULL, NULL);
     fail_unless (SIGNON_IS_IDENTITY (identity));
     methods = g_hash_table_new (g_str_hash, g_str_equal);
@@ -605,6 +762,8 @@ START_TEST(test_get_existing_identity)
     g_type_init ();
 
     g_debug("%s", G_STRFUNC);
+
+    main_loop = g_main_loop_new (NULL, FALSE);
     guint id = new_identity();
 
     fail_unless (id != 0);
@@ -616,7 +775,6 @@ START_TEST(test_get_existing_identity)
                  "Failed to initialize the Identity.");
 
     g_timeout_add (1000, identity_registered_cb, identity);
-    main_loop = g_main_loop_new (NULL, FALSE);
     g_main_loop_run (main_loop);
 
     end_test ();
@@ -686,6 +844,7 @@ START_TEST(test_store_credentials_identity)
     fail_unless (SIGNON_IS_IDENTITY (idty),
                  "Failed to initialize the Identity.");
 
+    main_loop = g_main_loop_new (NULL, FALSE);
     gint last_id = new_identity();
 
     GHashTable *methods = create_methods_hashtable();
@@ -704,7 +863,6 @@ START_TEST(test_store_credentials_identity)
     g_hash_table_destroy (methods);
 
     g_timeout_add (1000, test_quit_main_loop_cb, idty);
-    main_loop = g_main_loop_new (NULL, FALSE);
     g_main_loop_run (main_loop);
 
     g_object_unref(idty);
@@ -1119,6 +1277,8 @@ START_TEST(test_signout_identity)
 
     g_object_unref (idty);
     g_object_unref (idty2);
+
+    end_test ();
 }
 END_TEST
 
@@ -1156,6 +1316,8 @@ START_TEST(test_unregistered_identity)
 
     g_object_unref (idty);
     g_object_unref (idty2);
+
+    end_test ();
 }
 END_TEST
 
@@ -1269,8 +1431,10 @@ START_TEST(test_regression_unref)
                                  session_data,
                                  "mech1",
                                  test_regression_unref_process_cb,
-                                 g_strdup ("Hi there!"));
+                                 "Hi there!");
     g_main_loop_run (main_loop);
+
+    end_test ();
 }
 END_TEST
 
@@ -1296,6 +1460,8 @@ signon_suite(void)
     tcase_add_test (tc_core, test_auth_session_query_mechanisms);
     tcase_add_test (tc_core, test_auth_session_query_mechanisms_nonexisting);
     tcase_add_test (tc_core, test_auth_session_process);
+    tcase_add_test (tc_core, test_auth_session_process_failure);
+    tcase_add_test (tc_core, test_auth_session_process_after_store);
     tcase_add_test (tc_core, test_store_credentials_identity);
     tcase_add_test (tc_core, test_verify_secret_identity);
     tcase_add_test (tc_core, test_remove_identity);
